#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
using std::cout;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#endif

#include "floatImage.h"
#include "ucharImage.h"
#include "PGMFile.h"
#include "dct.h"
#include "Timer.h"
#include "Arguments.h"

// Compare execution times in seconds for simple and fast DCT. Output is suitable 
// for plotting using gnuplot
int main(int argc, char *argv[])
{
  CLArg param[] = {{"ifile","input_file"},
                   {0,0}};
  CLArg optpm[] = {{0,0}};
  CLArg flags[] = {{0,0}};
  Arguments args(argc, argv, param, optpm, flags);

  //Define unsigned char image and load from PGM format file
  ucharImage uci;
  if (!uci.load(PGMFile(args.arg("ifile"))))
    {
      cerr << "Error: could not load file " << args.arg("ifile") << endl;
      return 1;
    }

  cout.setf(ios::fixed);
  cout.setf(ios::showpoint);
  cout << "#   n        slow        fast\n";
  //Define float image and assign from unsigned char image converted to float
  floatImage fi = uci;
  unsigned int maxsize = min(fi.iwidth(), fi.iheight());
  for (unsigned int size = 8; size <= maxsize; size*=2)
    {
      //Define timer object
      Clock_CPU timer;
      //Define subimage and initialise it to a size x size subimage
      //(origin in upper left corner) of fi
      floatImage subimage = fi(0,0,size,size);
      //Define dctimage as a size x size float image
      floatImage dctimage(size,size);
      //Start timing an interval
      timer.mark();
      //Perform slow DCT
      dct(subimage,dctimage);
      //Read elapsed time
      float slow = timer.interval();
      //Construct fast DCT object
      ChanDCT chan(size);
      //Start timing an interval
      timer.mark();
      //Perform fast DCT
      chan.dct(subimage,dctimage);
      //Read elapsed time
      float fast = timer.interval();
      cout << setw(5) << size
	   << setw(12) << setprecision(2) << slow
	   << setw(12) << setprecision(2) << fast << endl;
    }

return 0;
}
