#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
using std::cout;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#endif

#include "floatVector.h"
#include "WvltFilters.h"
#include "CoefAccess.h"

//Illustrate use of CoefAccess object to index elements of a wavelet
//transformed vector as nodes in the tree of detail coefficients
int main()
{
  //Define wavelet object and initialise to Daubechies 4 wavelet
  Wavelet wavelet = Daubechies4;
  floatVector fv(8), trans;

  //Initialise vector fv
  for (unsigned int n = 0; n < 8; n++)
    fv(n) = cos(n*PI/7.0);

  //Set trans to the wavelet transform of fv
  trans = wavelet.fwt(fv);

  cout.setf(ios::fixed);
  cout.setf(ios::showpoint);
  //Include setfill('\n') in the stream to display as a column
  cout << "Vector      " << setw(6) << setprecision(2)  << fv << endl;
  cout << "FWT         " << setw(6) << setprecision(2)  << trans << endl;
  
  //Attach detail coefficient access object to wavelet transform vector
  CoefAccess d(trans);
  //Set i to the index of the highest resolution subband in the decomposition
  unsigned int i = d.depth() - 1;
  //Set smax to the number of coefficients in subband i
  unsigned int smax = CoefAccess::subband_length(i);
  //Zero all detail coefficients in subband i
  for (unsigned int j = 0; j < smax; j++)
    d(i,j) = 0.0;

  cout << "Modified FWT" << setw(6) << setprecision(2)  << trans << endl;
  
  //Set fv to the inverse wavelet transform of trans
  fv = wavelet.ifwt(trans);

  cout << "Inverse FWT " << setw(6) << setprecision(2)  << fv << endl;

  return 0;
}

