#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
using std::cout;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#endif

#include "ucharImage.h"
#include "ImageIterate.h"
#include "PGMFile.h"
#include "ViffFile.h"

// Illustrate file access interface - in particular the different
// possible ways of loading and saving images
int main()
{
  //Define 128 x 128 image with unsigned char pixels initialised to 0
  ucharImage a(128, 128, 0);

  //Define zig-zag scan iterator, for an image with the same size as image a
  ZigzagIterate imit(a.size());

  //Initialise pixel values
  unsigned int pv = 0;
  while (imit.state() != ImageIterate::stop)
      a(imit++) = pv++;

  //Save a in PGM format in file ftest.pgm
  if (!a.save(PGMFile("ftest.pgm")))
    {
      cerr << "Error writing file ftest.pgm\n";
      return 1;
    }

  //Define null (zero size) unsigned char image
  ucharImage b;

  //Load b in PGM Format from ftest.pgm
  if (!b.load(PGMFile("ftest.pgm")))
    {
      cerr << "Error reading file ftest.pgm\n";
      return 2;
    }

  //Create PGM format file interface and attach it to file ftest.pgm
  PGMFile pfi("ftest.pgm");

  //Check whether the file is readable
  if (pfi.readable())
    cerr << "File ftest.pgm is readable\n";
  else
    cerr << "File ftest.pgm is not readable\n";

  //Check whether the file is writable
  if (pfi.writable())
    cerr << "File ftest.pgm is writable\n";
  else
    cerr << "File ftest.pgm is not writable\n";

  //Check whether the file is in PGM format
  if (pfi.format_OK())
    cerr << "File ftest.pgm has valid PGM format\n";
  else
    cerr << "File ftest.pgm has invalid PGM format\n";

  //Display the size of the image
  cout << "Image size of image in file ftest.pgm is " 
       << pfi.isize() << endl;

  //Define image c and initialise each pixel to twice the value in the 
  //corresponding pixel in b
  ucharImage c = real(2)*b;

  //Save image c to the file to which pfi is attached, ie. ftest.pgm
  if (!pfi.save(c))
    {
      cerr << "Error writing file ftest.pgm\n";
      return 1;
    }

  //Load the image in file ftest.pgm into image b
  if (!pfi.load(b))
    {
      cerr << "Error reading file ftest.pgm\n";
      return 2;
    }

  //Create VIFF format file interface and attach it to file ftest.pgm
  ViffFile vfi("ftest.pgm");

  //Check whether the file is in VIFF format
  if (vfi.format_OK())
    cerr << "File ftest.pgm has valid VIFF format\n";
  else
    cerr << "File ftest.pgm has invalid VIFF format\n";

  return 0;
}

