#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <iomanip>
#include <fstream>
using std::cout;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#endif

#include "floatVector.h"
#include "Sequence.h"
#include "WvltFilters.h"
#include "Arguments.h"

#include <stdio.h>

//Plot scaling functions and wavelets
int main(int argc, char *argv[])
{
  CLArg param[] = {{0,0}};
  CLArg optpm[] = {{"wavnum", "index -of-wavelet-basis"},
		   {0,0}};
  CLArg flags[] = {{"file", "file-prefix"},
		   {"l", 0},
		   {0,0}};

  Arguments args(argc, argv, param, optpm, flags);

  if (args.value("l")) //List available wavelets
    {
      unsigned int i = 0;
      //List of predefined wavelets is terminated by a structure with
      //null pointer fields
      while (predefined_wavelets[i].wavelet)
	{
	  cout << setw(3) << i << "  " 
	       << predefined_wavelets[i].description
	       << endl;
	  i++;
	}
    }

  //Count available wavelets
  unsigned int numdefd = 0;
  while (predefined_wavelets[numdefd++].wavelet);
  numdefd--;
  unsigned int wvltidx = 0;
  args.value("wavnum", wvltidx);
  if (wvltidx >= numdefd)
    {
      cout << "Index must be less than " << numdefd << endl;
      exit(1);
    }

  //Set default prefix string for filename to wavelet description string
  char *prefix = predefined_wavelets[wvltidx].description;
  if (args.value("file"))
    prefix = args.arg("file");

  Wavelet* wavelet = predefined_wavelets[wvltidx].wavelet;
  cout << "Using wavelet: " << predefined_wavelets[wvltidx].description
       << endl;

  cout.setf(ios::fixed);
  cout.setf(ios::showpoint);
  //Print coefficients of filters making up the wavelet definition
  cout << *predefined_wavelets[wvltidx].wavelet << endl;

  char filename[128];
  sprintf(filename,"%s.H", prefix);
  ofstream Hs(filename, ios::out);
  Hs.setf(ios::fixed);
  Hs.setf(ios::showpoint);
  //Write analysis scaling function 
  Hs << wavelet->phi() << endl;

  sprintf(filename,"%s.G", prefix);
  ofstream Gs(filename, ios::out);
  Gs.setf(ios::fixed);
  Gs.setf(ios::showpoint);
  //Write analysis wavelet 
  Gs << wavelet->theta() << endl;

  sprintf(filename,"%s.H~", prefix);
  ofstream Hts(filename, ios::out);
  Hts.setf(ios::fixed);
  Hts.setf(ios::showpoint);
  //Write synthesis scaling function 
  Hts << wavelet->tphi() << endl;

  sprintf(filename,"%s.G~", prefix);
  ofstream Gts(filename, ios::out);
  Gts.setf(ios::fixed);
  Gts.setf(ios::showpoint);
  //Write synthesis wavelet
  Gts << wavelet->ttheta() << endl;

  return 0;
}

