// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: ImageIterate.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/


#include "ImageIterate.h"


void ImageIterate::reset(void)
{
  set();
  iistate = initial;
}


RasterIterate::State RasterIterate::next(void)
{
  switch (iistate) {
  case initial:
    iistate = running;
  case running:
    if (size.rightboundary(*this))
      {
	x = 0;
	y++;
      }
    else
      x++;
    if (size.rightboundary(*this) && size.bottomboundary(*this))
      iistate = final;
    break;
  case final:
    iistate = stop;
  case stop:
    break;
  }
  return iistate;
}



AlternatingRasterIterate::State AlternatingRasterIterate::next(void)
{
  switch (iistate) {
  case initial:
    iistate = running;
  case running:
    switch(arstate) {
    case forwards:
      if (size.rightboundary(*this))
	{
	  arstate = backwards;
	  y++;
	}
      else
	x++;
      break;
    case backwards:
      if (size.leftboundary(*this))
	{
	  arstate = forwards;
	  y++;
	}
      else
	x--;
      break;
    }
    if (size.bottomboundary(*this) &&
	((arstate == forwards && size.rightboundary(*this)) ||
	 (arstate == backwards && size.leftboundary(*this))))
      iistate = final;
    break;
  case final:
    iistate = stop;
  case stop:
    break;
  }
  return iistate;
}



ZigzagIterate::State ZigzagIterate::next(void)
{
  switch(iistate) {
  case initial:               //initial state
    iistate = running;
  case running:
    switch(zstate) {
    case horizontal:
      x++;
      if (size.rightboundary(*this) && size.bottomboundary(*this))
	iistate = final;
      else
	if (size.topboundary(*this))
	  zstate = downdiagonal;
	else
	  zstate = updiagonal;
      break;
    case vertical:
      y++;
      if (size.rightboundary(*this) && size.bottomboundary(*this))
	iistate = final;
      else
	if (size.leftboundary(*this))
	  zstate = updiagonal;
	else
	  zstate = downdiagonal;
      break;
    case downdiagonal:
      x--; y++;
      if (size.bottomboundary(*this))
	zstate = horizontal;
      else
	if (size.leftboundary(*this))
	  zstate = vertical;
      break;
    case updiagonal:
      x++; y--;
      if (size.rightboundary(*this))
	zstate = vertical;
      else
	if (size.topboundary(*this))
	  zstate = horizontal;
      break;
    }
    break;
  case final:
    iistate = stop;
  case stop:
    break;
  }
  return iistate;
}
