// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: PGMFile.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

*/


#include <ctype.h>
#include <string.h>

#include "PGMFile.h"


int PGMFile::skip_comment(ifstream &file)
{
    char c;
  
    do {
	file.get(c);
    } while (isspace(c));
  
    if (c == '#')  //Check for comment line
	{
	    do {
		file.get(c);
	    } while (c != 0x0A);
	    return(1);
	}
    else {
#if defined(HAVE_ISTREAM_UNGET)
	file.unget();
#else
	file.putback(c);
#endif
	return(0);
    }
}

int PGMFile::skip_comments(ifstream &file)
{
  while ( skip_comment(file) );
  return 0;
}

bool PGMFile::read_magic(ifstream &file)
{
  char buf[5];
  file.getline(buf, 4);
  return (strcmp(buf, "P5") == 0) ? true : false;
}

ImageSize PGMFile::read_size(ifstream &file)
{
  unsigned int load_width = 0, load_height = 0;
  skip_comments(file);
  file >> load_width >> load_height;
  return ImageSize(load_width, load_height);
}

unsigned long int PGMFile::read_pixmax(ifstream &file)
{
  unsigned long int load_pix_max = 0;
  skip_comments(file);
  file >> load_pix_max;
  return load_pix_max;
}


bool PGMFile::load(Image<unsigned char, real>& im) const
{
  if (!filename() || filename()[0] == '\0')
    return false;
  ifstream imf(filename(), IOS_IN);
  if (!STREAM_OPEN(imf))
    return false;
  if (!read_magic(imf))
    return false;
  
  ImageSize ls = read_size(imf);
  if (ls.null())
    return false;
  else
    im.reset(ls);
    
  if (read_pixmax(imf) > 255)
    return false;
 
  imf.ignore(1, (char)10);

  if (!im.raw_read(imf))
    return false;

  imf.close();
  return true;
}


bool PGMFile::save(const Image<unsigned char, real>& im) const
{
  if (!filename() || filename()[0] == '\0')
    return false;
  ofstream imf(filename(), IOS_OUT);
  if (!STREAM_OPEN(imf))
    return false;

  imf << "P5\n" << im.iwidth() << "  " << im.iheight()
      << endl << 255 << endl;
  if (!imf)
    return false;

  if (!im.raw_write(imf))
    return false;

  imf.close();
  return true;
}

bool PGMFile::format_OK() const 
{
  ifstream imfs(filename(), IOS_IN);
  if (!STREAM_OPEN(imfs))
    return false;

  bool status = read_magic(imfs);
  imfs.close();
  return status;
}

ImageSize PGMFile::isize() const
{
  ImageSize is(0,0);
  ifstream imfs(filename(), IOS_IN);
  if (!STREAM_OPEN(imfs))
    return is;

  if (!read_magic(imfs))
    return is;
  is = read_size(imfs);
  imfs.close();
  return is;
}


