// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: PGMFile.h,v 1.2 2005/01/29 15:15:15 brendt Exp $

 */
 
#ifndef _PGMFile_h
#define _PGMFile_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
using std::endl;
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "ImageFile.h"
#include "ucharImage.h"


//! unit="Image and Vector Types"
//! module="File Access"
//! level = 3

//: PGM file format handler - NB only handles raw 8 bit pgm
class PGMFile : public ImageFileInterface<unsigned char, real>,
                public ImageFile
{
private:
  static int skip_comment(ifstream &file);
  static int skip_comments(ifstream &file);
  static bool read_magic(ifstream &file);
  static ImageSize read_size(ifstream &file);
  static long unsigned int read_pixmax(ifstream &file);
public:
  //Construct given filename
  PGMFile(char *name) : ProtoVectorFile(name), ImageFile(name) {}
  //Load PGM image
  bool load(Image<unsigned char, real>& im) const;
  //Save PGM image
  bool save(const Image<unsigned char, real>& im) const;

  //Check header format
  bool format_OK() const;
  //Return image size
  ImageSize isize() const;
};

#endif
