// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: VectorSize.h,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/


#ifndef _VectorSize_h
#define _VectorSize_h

//! unit="Image and Vector Types"
//! module="Vector Support Classes"
//! level = 1

//: Vector dimensions
// VectorSize represents the length of a vector. Indices within the
// vector range between 0 and length-1
class VectorSize {
private:
  unsigned int length;
public:
  VectorSize(void) { reset(); }
  VectorSize(unsigned int l) { length = l; }
  VectorSize(const VectorSize& vs) { length = vs.length; }
  void reset() { length = 0; }
  // return number of elements in vector (same as vlength)
  unsigned int elements() const { return length; }
  // Return vector length
  unsigned int vlength() const { return length; }
  // Equality test
  bool operator==(const VectorSize& vs) const;
  // Inequality test
  bool operator!=(const VectorSize& vs) const;
};

inline bool VectorSize::operator==(const VectorSize& vs) const
{               
  return bool(length == vs.length); 
}

inline bool VectorSize::operator!=(const VectorSize& vs) const
{ 
  return bool(length != vs.length); 
}

#endif
