// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: ViffFile.cc,v 1.2 2005/01/28 23:55:09 brendt Exp $

*/



#include <ctype.h>
#include "ViffFile.h"


/* The following definitions follow those in files viff.h and
   vdefines.h in the Khoros 1 distribution, which is Copyright 1990,
   University of New Mexico.  */

#define XV_FILE_MAGIC_NUM   0xab
#define XV_FILE_TYPE_XVIFF  1
#define VIFF_HEADERSIZE     1024 /* This should be 4096 on a Cray */

#define	VFF_TYP_BIT		0	/* pixels are binary */
#define	VFF_TYP_1_BYTE		1	/* pixels are byte (unsigned char) */
#define	VFF_TYP_2_BYTE		2	/* pixels are two byte (short int) */
#define	VFF_TYP_4_BYTE		4	/* pixels are four byte (integer) */
#define	VFF_TYP_FLOAT		5	/* pixels are float */
#define	VFF_TYP_COMPLEX		6	/* pixels are complex float */
#define VFF_TYP_DOUBLE		9	/* pixels are double */
#define VFF_TYP_DCOMPLEX       10	/* double complex */

#define VFF_MAP_OPTIONAL	   1

#define	VFF_DEP_IEEEORDER   	0x2	/* IEEE byte ordering */
#define	VFF_DEP_DECORDER    	0x4	/* DEC (VAX) byte ordering */
#define VFF_DEP_NSORDER		0x8	/* NS32000 byte ordering */
#define VFF_DEP_CRAYORDER	0xA	/* Cray byte size and ordering */
#define	VFF_DEP_BIGENDIAN	VFF_DEP_IEEEORDER
#define	VFF_DEP_LITENDIAN	VFF_DEP_NSORDER

#define XV_IMAGE_REL_NUM	   1
#define XV_IMAGE_VER_NUM	   3
#define VFF_NOTSUB   	   ~0
#define VFF_LOC_IMPLICIT    1
#define VFF_DES_RAW	   0
#define VFF_MS_NONE	   0
#define VFF_MAPTYP_NONE	   0
#define VFF_CM_NONE	   0

struct xvimage	{
  unsigned char identifier, file_type, release, version,
    machine_dep, trash[3], comment[512];	
  unsigned long row_size, col_size, subrow_size;    
  long startx, starty;
  float pixsizx, pixsizy;
  unsigned long location_type, location_dim, num_of_images,
    num_data_bands, data_storage_type, data_encode_scheme,
    map_scheme, map_storage_type, map_row_size,
    map_col_size, map_subrow_size, map_enable,
    maps_per_cycle, color_space_model;
  unsigned long ispare1,ispare2;
  float fspare1,fspare2;
  char reserve[VIFF_HEADERSIZE - (21*sizeof(long))  
	      - (520*sizeof(char)) 
	      - (2*sizeof(char *)) - (4*sizeof(float))
	      - (sizeof(float *))];	
  char *maps;
  float *location;
  char *imagedata;
};


/* End of Khoros definitions */


#ifndef NO_MEMBER_TEMPLATES
template <class T>
class VIFFDataType {
 public:
  static unsigned long code;
};
// Warning: the following make assumptions about fundamental type sizes
template <>
unsigned long VIFFDataType<unsigned char>::code = VFF_TYP_1_BYTE;
template <>
unsigned long VIFFDataType<unsigned short>::code = VFF_TYP_2_BYTE;
template <>
unsigned long VIFFDataType<unsigned int>::code = VFF_TYP_4_BYTE;
template <>
unsigned long VIFFDataType<float>::code = VFF_TYP_FLOAT;
template <>
unsigned long VIFFDataType<double>::code = VFF_TYP_DOUBLE;
#endif


ViffFile::ViffFile(char *name) : 
  ProtoVectorFile(name), ImageFile(name), VectorFile(name)  {} 


bool ViffFile::read_header(ifstream &fstr, xvimage& hdr)
{
  return (bool)(void*)fstr.read((char *)&hdr, sizeof(xvimage));
}

bool ViffFile::write_header(ofstream &fstr, xvimage& hdr)
{
  return (bool)(void*)fstr.write((char *)&hdr, sizeof(xvimage));
}


#ifdef NO_MEMBER_TEMPLATES

bool ViffFile::load(Image<unsigned char, real>& im) const
{
  if (!filename() || filename()[0] == '\0')
    return false;
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VFF_TYP_1_BYTE))
    return false;
  else
    im.reset(ls);

  if (!im.raw_read(ifs))
    return false;
  
  ifs.close();
  return true;
}


bool ViffFile::save(const Image<unsigned char, real>& im) const
{
  if (!filename() || filename()[0] == '\0')
    return false;
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  set_header(hdr, im.size(), VFF_TYP_1_BYTE);
  if (!write_header(ofs, hdr))
    return false;

  if (!im.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}


bool ViffFile::load(Image<unsigned short, real>& im) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VFF_TYP_2_BYTE))
    return false;
  else
    im.reset(ls);

  if (!im.raw_read(ifs))
    return false;

  ifs.close();
  return true;
}


bool ViffFile::save(const Image<unsigned short, real>& im) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  set_header(hdr, im.size(), VFF_TYP_2_BYTE);
  if (!write_header(ofs, hdr))
    return false;

  if (!im.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}



bool ViffFile::load(Image<float, float>& im) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VFF_TYP_FLOAT))
    return false;
  else
    im.reset(ls);

  if (!im.raw_read(ifs))
    return false;

  ifs.close();
  return true;
}


bool ViffFile::save(const Image<float, float>& im) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  set_header(hdr, im.size(), VFF_TYP_FLOAT);
  if (!write_header(ofs, hdr))
    return false;

  if (!im.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}


bool ViffFile::load(Vector<unsigned char, real>& v) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VFF_TYP_1_BYTE))
    return false;
  else
    v.reset(ls.iwidth());

  if (!v.raw_read(ifs))
    return false;

  ifs.close();
  return true;
}


bool ViffFile::save(const Vector<unsigned char, real>& v) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  ImageSize is(v.vlength(), 1);
  set_header(hdr, is, VFF_TYP_1_BYTE);
  if (!write_header(ofs, hdr))
    return false;

  if (!v.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}


bool ViffFile::load(Vector<unsigned short, real>& v) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VFF_TYP_2_BYTE))
    return false;
  else
    v.reset(ls.iwidth());

  if (!v.raw_read(ifs))
    return false;

  ifs.close();
  return true;
}


bool ViffFile::save(const Vector<unsigned short, real>& v) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  ImageSize is(v.vlength(), 1);
  set_header(hdr, is, VFF_TYP_2_BYTE);
  if (!write_header(ofs, hdr))
    return false;

  if (!v.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}


bool ViffFile::load(Vector<float, float>& v) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;

  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VFF_TYP_FLOAT))
    return false;
  else
    v.reset(ls.iwidth());

  if (!v.raw_read(ifs))
    return false;

  ifs.close();
  return true;
}


bool ViffFile::save(const Vector<float, float>& v) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  ImageSize is(v.vlength(), 1);
  set_header(hdr, is, VFF_TYP_FLOAT);
  if (!write_header(ofs, hdr))
    return false;

  if (!v.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}

#else

bool ViffFile::load(Image<unsigned char, real>& im) const
{
  return vload(im);
}


bool ViffFile::save(const Image<unsigned char, real>& im) const
{
  return vsave(im);
}


bool ViffFile::load(Image<unsigned short, real>& im) const
{
  return vload(im);
}


bool ViffFile::save(const Image<unsigned short, real>& im) const
{
  return vsave(im);
}


bool ViffFile::load(Image<float, float>& im) const
{
  return vload(im);
}


bool ViffFile::save(const Image<float, float>& im) const
{
  return vsave(im);
}


bool ViffFile::load(Vector<unsigned char, real>& v) const
{
  return vload(v);
}


bool ViffFile::save(const Vector<unsigned char, real>& v) const
{
  return vsave(v);
}


bool ViffFile::load(Vector<unsigned short, real>& v) const
{
  return vload(v);
}


bool ViffFile::save(const Vector<unsigned short, real>& v) const
{
  return vsave(v);
}


bool ViffFile::load(Vector<float, float>& v) const
{
  return vload(v);
}


bool ViffFile::save(const Vector<float, float>& v) const
{
  return vsave(v);
}


template <class T, class R>
bool ViffFile::vload(Vector<T, R>& v) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VIFFDataType<T>::code))
    return false;
  else
    v.reset(ls.iwidth());

  if (!v.raw_read(ifs))
    return false;

  ifs.close();
  return true;
}

template <class T, class R>
bool ViffFile::vsave(const Vector<T, R>& v) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  ImageSize is(v.vlength(), 1);
  set_header(hdr, is, VIFFDataType<T>::code);
  if (!write_header(ofs, hdr))
    return false;

  if (!v.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}

template <class T, class R>
bool ViffFile::vload(Image<T, R>& v) const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  if (!read_header(ifs, hdr))
    return false;

  ImageSize ls;
  if (!get_header(hdr, ls, VIFFDataType<T>::code))
    return false;
  else
    v.reset(ls);

  if (!v.raw_read(ifs))
    return false;
  
  ifs.close();
  return true;
}

template <class T, class R>
bool ViffFile::vsave(const Image<T, R>& v) const
{
  ofstream ofs(filename(), IOS_OUT);
  if (!STREAM_OPEN(ofs))
    return false;

  xvimage hdr;
  set_header(hdr, v.size(), VIFFDataType<T>::code);
  if (!write_header(ofs, hdr))
    return false;

  if (!v.raw_write(ofs))
    return false;

  ofs.close();
  return true;
}

#endif

bool ViffFile::format_OK() const 
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return false;
  
  xvimage hdr;
  int status = read_header(ifs, hdr);
  ifs.close();
  return bool(status && header_OK(hdr));
}

bool ViffFile::header_OK(xvimage& hdr) //Could be more careful
{
  return bool(((unsigned char)hdr.identifier == XV_FILE_MAGIC_NUM) &&
    (hdr.file_type == XV_FILE_TYPE_XVIFF) &&
    (hdr.num_data_bands == 1)             &&
    (hdr.map_enable == VFF_MAP_OPTIONAL));
}

ImageSize ViffFile::isize() const
{
  ImageSize is(0,0);
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return is;

  xvimage hdr;
  if (!read_header(ifs, hdr))
    return is;
  is = ImageSize(hdr.row_size, hdr.col_size);
  ifs.close();
  return is;
}

VectorSize ViffFile::vsize() const
{
  ifstream ifs(filename(), IOS_IN);
  if (!STREAM_OPEN(ifs))
    return VectorSize(0);

  xvimage hdr;
  if (!read_header(ifs, hdr))
    return VectorSize(0);
  ifs.close();
  return VectorSize(hdr.row_size);
  //Should check that hdr.col_size == 1
}

void ViffFile::set_header(xvimage& hdr, const ImageSize& imsize,
			  unsigned long storage_type)
{
  hdr.identifier = XV_FILE_MAGIC_NUM;
  hdr.file_type = XV_FILE_TYPE_XVIFF;
  hdr.release = XV_IMAGE_REL_NUM;
  hdr.version = XV_IMAGE_VER_NUM;
#if defined(WORDS_BIGENDIAN)
  hdr.machine_dep = VFF_DEP_BIGENDIAN; 
#else
  hdr.machine_dep = VFF_DEP_LITENDIAN;
#endif
  hdr.row_size = imsize.iwidth();
  hdr.col_size = imsize.iheight();
  hdr.subrow_size = 0;
  hdr.startx = hdr.starty = VFF_NOTSUB;
  hdr.pixsizx = hdr.pixsizy = 0;
  hdr.location_type = VFF_LOC_IMPLICIT;
  hdr.location_dim = 0;
  hdr.num_of_images = 1;
  hdr.num_data_bands = 1;
  hdr.data_storage_type = storage_type;
  hdr.data_encode_scheme = VFF_DES_RAW;
  hdr.map_scheme = VFF_MS_NONE;
  hdr.map_storage_type = VFF_MAPTYP_NONE;
  hdr.map_row_size = hdr.map_col_size = 0;
  hdr.map_subrow_size = 0;
  hdr.map_enable = VFF_MAP_OPTIONAL;
  hdr.maps_per_cycle = 0;
  hdr.color_space_model = VFF_CM_NONE;
  hdr.ispare1 = hdr.ispare2 = 0;
  hdr.fspare1 = hdr.fspare2 = 0;
  hdr.maps = 0;
  hdr.location = 0;
  hdr.imagedata = 0;
}

bool ViffFile::get_header(xvimage& hdr, ImageSize& imsize,
			  unsigned long storage_type)
{
  imsize = ImageSize(hdr.row_size, hdr.col_size);
  return bool(header_OK(hdr) && !imsize.null() &&
         hdr.data_storage_type == storage_type);
}



#if !defined(NO_MEMBER_TEMPLATES) && defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template bool ViffFile::vload(Vector<unsigned char, real>& v) const;
template bool ViffFile::vsave(const Vector<unsigned char, real>& v) const;
template bool ViffFile::vload(Image<unsigned char, real>& v) const;
template bool ViffFile::vsave(const Image<unsigned char, real>& v) const;

template bool ViffFile::vload(Vector<unsigned short, real>& v) const;
template bool ViffFile::vsave(const Vector<unsigned short, real>& v) const;
template bool ViffFile::vload(Image<unsigned short, real>& v) const;
template bool ViffFile::vsave(const Image<unsigned short, real>& v) const;

template bool ViffFile::vload(Vector<float, float>& v) const;
template bool ViffFile::vsave(const Vector<float, float>& v) const;
template bool ViffFile::vload(Image<float, float>& v) const;
template bool ViffFile::vsave(const Image<float, float>& v) const;
#endif
