// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image
    processing. 

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: ViffFile.h,v 1.1 2003/02/01 17:46:23 brendt Exp $

 */

 
#ifndef _ViffFile_h
#define _ViffFile_h

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
using std::ofstream;
using std::ifstream;
#else
#include <iostream.h>
#include <fstream.h>
#endif

#include "ImageFile.h"
#include "floatImage.h"
#include "floatVector.h"
#include "ucharImage.h"
#include "ushortImage.h"


//! unit="Image and Vector Types"
//! module="File Access"
//! level = 3

struct xvimage; //incomplete declaration of Khoros VIFF structure

//: VIFF file format handler
class ViffFile : public ImageFileInterface<unsigned char, real>,
                 public ImageFileInterface<unsigned short, real>,
                 public ImageFileInterface<float, float>,
		 public VectorFileInterface<unsigned char, real>,
		 public VectorFileInterface<unsigned short, real>,
		 public VectorFileInterface<float, float>,
		 public ImageFile, public VectorFile
{
private:
  static bool read_header(ifstream &file, xvimage& hdr);
  static bool write_header(ofstream &file, xvimage& hdr);
  static bool header_OK(xvimage& hdr);
  static void set_header(xvimage& hdr, const ImageSize& imsize,
			 unsigned long storage_type);
  static bool get_header(xvimage& hdr, ImageSize& imsize,
			unsigned long storage_type);
public:
  ViffFile(char *name);

  //Load VIFF unsigned char image
  bool load(Image<unsigned char, real>& im) const;
  //Save VIFF unsigned char image
  bool save(const Image<unsigned char, real>& im) const;

  //Load VIFF unsigned short image
  bool load(Image<unsigned short, real>& im) const;
  //Save VIFF unsigned short image
  bool save(const Image<unsigned short, real>& im) const;
  
  //Load VIFF float image
  bool load(Image<float, float>& im) const;
  //Save VIFF float image
  bool save(const Image<float, float>& im) const;

  //Load VIFF unsigned char vector
  bool load(Vector<unsigned char, real>& im) const;
  //Save VIFF unsigned char vector
  bool save(const Vector<unsigned char, real>& im) const;

  //Load VIFF unsigned short vector
  bool load(Vector<unsigned short, real>& im) const;
  //Save VIFF unsigned short vector
  bool save(const Vector<unsigned short, real>& im) const;

  //Load VIFF float vector
  bool load(Vector<float, float>& im) const;
  //Save VIFF float vector
  bool save(const Vector<float, float>& im) const;

#ifndef NO_MEMBER_TEMPLATES
  //Load VIFF vector
  template <class T, class R>
    bool vload(Vector<T, R>& v) const;
  //Save VIFF vector
  template <class T, class R>
    bool vsave(const Vector<T, R>& v) const;
  //Load VIFF image
  template <class T, class R>
    bool vload(Image<T, R>& v) const;
  //Save VIFF image
  template <class T, class R>
    bool vsave(const Image<T, R>& v) const;
#endif  


  //Check header format
  bool format_OK() const;
  //Return image size
  ImageSize isize() const;
  //Return vector size
  VectorSize vsize() const;
};

#endif
