// -*- Mode:c++ -*-
/******************************COPYRIGHT***********************************
  
    This file forms a part of ImageLib, a C++ class library for image
    processing. 

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************COPYRIGHT***********************************/

/*

  $Id: instant.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $
  
*/

#include "ucharVector.h"
#include "ushortVector.h"
#include "floatVector.h"
#include "ucharImage.h"
#include "ushortImage.h"
#include "floatImage.h"

#if !defined(INCLUDE_TEMPLATE_DEFINITIONS)
#include "ProtoVector.cc"
#include "Vector.cc"
#include "Subvector.cc"
#include "Image.cc"
#include "Subimage.cc"
#endif

#if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template class ProtoVector<unsigned char, real, VectorSize>;
template void swap(ProtoVector<unsigned char, real, VectorSize>& u, ProtoVector<unsigned char, real, VectorSize>&);
template real inproduct(const ProtoVector<unsigned char, real, VectorSize>&, const ProtoVector<unsigned char, real, VectorSize>&);
template real MSE(const ProtoVector<unsigned char, real, VectorSize>&, const ProtoVector<unsigned char, real, VectorSize>&);
template real RMSE(const ProtoVector<unsigned char, real, VectorSize>&, const ProtoVector<unsigned char, real, VectorSize>&);
template real SNR(const ProtoVector<unsigned char, real, VectorSize>&, const ProtoVector<unsigned char, real, VectorSize>&);
template real PSNR(const ProtoVector<unsigned char, real, VectorSize>&, const ProtoVector<unsigned char, real, VectorSize>&);

template class Vector<unsigned char, real>;
template class VectorRef<unsigned char, real>;
template class Subvector<unsigned char, real>;
template class VectorFileInterface<unsigned char, real>;
template VectorRef<unsigned char, real> operator+(const Vector<unsigned char, real>&, const Vector<unsigned char, real>&);
template VectorRef<unsigned char, real> operator+(const Vector<unsigned char, real>&, unsigned char);
template VectorRef<unsigned char, real> operator-(const Vector<unsigned char, real>&, const Vector<unsigned char, real>&);
template VectorRef<unsigned char, real> operator-(const Vector<unsigned char, real>&, unsigned char);
template VectorRef<unsigned char, real> operator-(unsigned char, const Vector<unsigned char, real>&);
template VectorRef<unsigned char, real> operator*(const Vector<unsigned char, real>&, real);
template VectorRef<unsigned char, real> operator*(real, const Vector<unsigned char, real>&);
template VectorRef<unsigned char, real> operator/(const Vector<unsigned char, real>&, real);
template VectorRef<unsigned char, real> abs(const Vector<unsigned char, real>&);
template VectorRef<unsigned char, real> clip(const Vector<unsigned char, real>&, unsigned char, unsigned char);
template ImageRef<unsigned char, real> rowimage(const Vector<unsigned char, real>&);
template ImageRef<unsigned char, real> colimage(const Vector<unsigned char, real>&);
template VectorRef<unsigned char, real> subsample(const Vector<unsigned char, real>&, unsigned int);
template VectorRef<unsigned char, real> concatenate(const Vector<unsigned char, real>&, const Vector<unsigned char, real>&);
template ostream& operator<<(ostream &, const Vector<unsigned char, real>&);

// Conversions from other types
#if !defined(NO_MEMBER_TEMPLATES)
template ProtoVector<unsigned char, real, VectorSize>::ProtoVector(const ProtoVector<unsigned short, real, VectorSize>&);
template ProtoVector<unsigned char, real, VectorSize>::ProtoVector(const ProtoVector<float, float, VectorSize>&);
template Vector<unsigned char, real>::Vector(const Vector<unsigned short, real>&);
template Vector<unsigned char, real>::Vector(const Vector<float, float>&);
#endif /* NO_MEMBER_TEMPLATES */

#endif /* EXPLICIT_TEMPLATE_INSTANTIATION */


#if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template class ProtoVector<unsigned char, real, ImageSize>;
template void swap(ProtoVector<unsigned char, real, ImageSize>& u, ProtoVector<unsigned char, real, ImageSize>&);
template real inproduct(const ProtoVector<unsigned char, real, ImageSize>&, const ProtoVector<unsigned char, real, ImageSize>&);
template real MSE(const ProtoVector<unsigned char, real, ImageSize>&, const ProtoVector<unsigned char, real, ImageSize>&);
template real RMSE(const ProtoVector<unsigned char, real, ImageSize>&, const ProtoVector<unsigned char, real, ImageSize>&);
template real SNR(const ProtoVector<unsigned char, real, ImageSize>&, const ProtoVector<unsigned char, real, ImageSize>&);
template real PSNR(const ProtoVector<unsigned char, real, ImageSize>&, const ProtoVector<unsigned char, real, ImageSize>&);

template class Image<unsigned char, real>;
template class ImageRef<unsigned char, real>;
template class Subimage<unsigned char, real>;
template class ImageFileInterface<unsigned char, real>;
template ImageRef<unsigned char, real> operator+(const Image<unsigned char, real>&, const Image<unsigned char, real>&);
template ImageRef<unsigned char, real> operator+(const Image<unsigned char, real>&, unsigned char);
template ImageRef<unsigned char, real> operator-(const Image<unsigned char, real>&, const Image<unsigned char, real>&);
template ImageRef<unsigned char, real> operator-(const Image<unsigned char, real>&, unsigned char);
template ImageRef<unsigned char, real> operator-(unsigned char, const Image<unsigned char, real>&);
template ImageRef<unsigned char, real> operator*(const Image<unsigned char, real>&, real);
template ImageRef<unsigned char, real> operator*(real, const Image<unsigned char, real>&);
template ImageRef<unsigned char, real> operator/(const Image<unsigned char, real>&, real);
template ImageRef<unsigned char, real> abs(const Image<unsigned char, real>&);
template ImageRef<unsigned char, real> clip(const Image<unsigned char, real>&, unsigned char, unsigned char);
template ImageRef<unsigned char, real> isometry(const Image<unsigned char, real>&, unsigned char);
template ImageRef<unsigned char, real> subsample(const Image<unsigned char, real>&, unsigned int, unsigned int);
template ostream& operator<<(ostream &, const Image<unsigned char, real>&);

// Conversions from other types
#if !defined(NO_MEMBER_TEMPLATES)
template ProtoVector<unsigned char, real, ImageSize>::ProtoVector(const ProtoVector<unsigned short, real, ImageSize>&);
template ProtoVector<unsigned char, real, ImageSize>::ProtoVector(const ProtoVector<float, float, ImageSize>&);
template Image<unsigned char, real>::Image(const Image<unsigned short, real>&);
template Image<unsigned char, real>::Image(const Image<float, float>&);
#endif /* NO_MEMBER_TEMPLATES */

#endif /* EXPLICIT_TEMPLATE_INSTANTIATION */

#if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template class ProtoVector<unsigned short, real, VectorSize>;
template void swap(ProtoVector<unsigned short, real, VectorSize>& u, ProtoVector<unsigned short, real, VectorSize>&);
template real inproduct(const ProtoVector<unsigned short, real, VectorSize>&, const ProtoVector<unsigned short, real, VectorSize>&);
template real MSE(const ProtoVector<unsigned short, real, VectorSize>&, const ProtoVector<unsigned short, real, VectorSize>&);
template real RMSE(const ProtoVector<unsigned short, real, VectorSize>&, const ProtoVector<unsigned short, real, VectorSize>&);
template real SNR(const ProtoVector<unsigned short, real, VectorSize>&, const ProtoVector<unsigned short, real, VectorSize>&);
template real PSNR(const ProtoVector<unsigned short, real, VectorSize>&, const ProtoVector<unsigned short, real, VectorSize>&);

template class Vector<unsigned short, real>;
template class VectorRef<unsigned short, real>;
template class Subvector<unsigned short, real>;
template class VectorFileInterface<unsigned short, real>;
template VectorRef<unsigned short, real> operator+(const Vector<unsigned short, real>&, const Vector<unsigned short, real>&);
template VectorRef<unsigned short, real> operator+(const Vector<unsigned short, real>&, unsigned short);
template VectorRef<unsigned short, real> operator-(const Vector<unsigned short, real>&, const Vector<unsigned short, real>&);
template VectorRef<unsigned short, real> operator-(const Vector<unsigned short, real>&, unsigned short);
template VectorRef<unsigned short, real> operator-(unsigned short, const Vector<unsigned short, real>&);
template VectorRef<unsigned short, real> operator*(const Vector<unsigned short, real>&, real);
template VectorRef<unsigned short, real> operator*(real, const Vector<unsigned short, real>&);
template VectorRef<unsigned short, real> operator/(const Vector<unsigned short, real>&, real);
template VectorRef<unsigned short, real> abs(const Vector<unsigned short, real>&);
template VectorRef<unsigned short, real> clip(const Vector<unsigned short, real>&, unsigned short, unsigned short);
template ImageRef<unsigned short, real> rowimage(const Vector<unsigned short, real>&);
template ImageRef<unsigned short, real> colimage(const Vector<unsigned short, real>&);
template VectorRef<unsigned short, real> subsample(const Vector<unsigned short, real>&, unsigned int);
template VectorRef<unsigned short, real> concatenate(const Vector<unsigned short, real>&, const Vector<unsigned short, real>&);
template ostream& operator<<(ostream &, const Vector<unsigned short, real>&);

// Conversions from other types
#if !defined(NO_MEMBER_TEMPLATES)
template ProtoVector<unsigned short, real, VectorSize>::ProtoVector(const ProtoVector<unsigned char, real, VectorSize>&);
template ProtoVector<unsigned short, real, VectorSize>::ProtoVector(const ProtoVector<float, float, VectorSize>&);
template Vector<unsigned short, real>::Vector(const Vector<unsigned char, real>&);
template Vector<unsigned short, real>::Vector(const Vector<float, float>&);
#endif /* NO_MEMBER_TEMPLATES */

#endif /* EXPLICIT_TEMPLATE_INSTANTIATION */


#if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template class ProtoVector<unsigned short, real, ImageSize>;
template void swap(ProtoVector<unsigned short, real, ImageSize>& u, ProtoVector<unsigned short, real, ImageSize>&);
template real inproduct(const ProtoVector<unsigned short, real, ImageSize>&, const ProtoVector<unsigned short, real, ImageSize>&);
template real MSE(const ProtoVector<unsigned short, real, ImageSize>&, const ProtoVector<unsigned short, real, ImageSize>&);
template real RMSE(const ProtoVector<unsigned short, real, ImageSize>&, const ProtoVector<unsigned short, real, ImageSize>&);
template real SNR(const ProtoVector<unsigned short, real, ImageSize>&, const ProtoVector<unsigned short, real, ImageSize>&);
template real PSNR(const ProtoVector<unsigned short, real, ImageSize>&, const ProtoVector<unsigned short, real, ImageSize>&);

template class Image<unsigned short, real>;
template class ImageRef<unsigned short, real>;
template class Subimage<unsigned short, real>;
template class ImageFileInterface<unsigned short, real>;
template ImageRef<unsigned short, real> operator+(const Image<unsigned short, real>&, const Image<unsigned short, real>&);
template ImageRef<unsigned short, real> operator+(const Image<unsigned short, real>&, unsigned short);
template ImageRef<unsigned short, real> operator-(const Image<unsigned short, real>&, const Image<unsigned short, real>&);
template ImageRef<unsigned short, real> operator-(const Image<unsigned short, real>&, unsigned short);
template ImageRef<unsigned short, real> operator-(unsigned short, const Image<unsigned short, real>&);
template ImageRef<unsigned short, real> operator*(const Image<unsigned short, real>&, real);
template ImageRef<unsigned short, real> operator*(real, const Image<unsigned short, real>&);
template ImageRef<unsigned short, real> operator/(const Image<unsigned short, real>&, real);
template ImageRef<unsigned short, real> abs(const Image<unsigned short, real>&);
template ImageRef<unsigned short, real> clip(const Image<unsigned short, real>&, unsigned short, unsigned short);
template ImageRef<unsigned short, real> isometry(const Image<unsigned short, real>&, unsigned char);
template ImageRef<unsigned short, real> subsample(const Image<unsigned short, real>&, unsigned int, unsigned int);
template ostream& operator<<(ostream &, const Image<unsigned short, real>&);

// Conversions from other types
#if !defined(NO_MEMBER_TEMPLATES)
template ProtoVector<unsigned short, real, ImageSize>::ProtoVector(const ProtoVector<unsigned char, real, ImageSize>&);
template ProtoVector<unsigned short, real, ImageSize>::ProtoVector(const ProtoVector<float, float, ImageSize>&);
template Image<unsigned short, real>::Image(const Image<unsigned char, real>&);
template Image<unsigned short, real>::Image(const Image<float, float>&);
#endif /* NO_MEMBER_TEMPLATES */

#endif /* EXPLICIT_TEMPLATE_INSTANTIATION */

#if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template class ProtoVector<float, float, VectorSize>;
template void swap(ProtoVector<float, float, VectorSize>& u, ProtoVector<float, float, VectorSize>&);
template float inproduct(const ProtoVector<float, float, VectorSize>&, const ProtoVector<float, float, VectorSize>&);
template float MSE(const ProtoVector<float, float, VectorSize>&, const ProtoVector<float, float, VectorSize>&);
template float RMSE(const ProtoVector<float, float, VectorSize>&, const ProtoVector<float, float, VectorSize>&);
template float SNR(const ProtoVector<float, float, VectorSize>&, const ProtoVector<float, float, VectorSize>&);
template float PSNR(const ProtoVector<float, float, VectorSize>&, const ProtoVector<float, float, VectorSize>&);

template class Vector<float, float>;
template class VectorRef<float, float>;
template class Subvector<float, float>;
template class VectorFileInterface<float, float>;
template VectorRef<float, float> operator+(const Vector<float, float>&, const Vector<float, float>&);
template VectorRef<float, float> operator+(const Vector<float, float>&, float);
template VectorRef<float, float> operator-(const Vector<float, float>&, const Vector<float, float>&);
template VectorRef<float, float> operator-(const Vector<float, float>&, float);
template VectorRef<float, float> operator-(float, const Vector<float, float>&);
template VectorRef<float, float> operator*(const Vector<float, float>&, float);
template VectorRef<float, float> operator*(float, const Vector<float, float>&);
template VectorRef<float, float> operator/(const Vector<float, float>&, float);
template VectorRef<float, float> abs(const Vector<float, float>&);
template VectorRef<float, float> clip(const Vector<float, float>&, float, float);
template ImageRef<float, float> rowimage(const Vector<float, float>&);
template ImageRef<float, float> colimage(const Vector<float, float>&);
template VectorRef<float, float> subsample(const Vector<float, float>&, unsigned int);
template VectorRef<float, float> concatenate(const Vector<float, float>&, const Vector<float, float>&);
template ostream& operator<<(ostream &, const Vector<float, float>&);

// Conversions from other types
#if !defined(NO_MEMBER_TEMPLATES)
template ProtoVector<float, float, VectorSize>::ProtoVector(const ProtoVector<unsigned char, real, VectorSize>&);
template ProtoVector<float, float, VectorSize>::ProtoVector(const ProtoVector<unsigned short, real, VectorSize>&);
template Vector<float, float>::Vector(const Vector<unsigned char, real>&);
template Vector<float, float>::Vector(const Vector<unsigned short, real>&);
#endif /* NO_MEMBER_TEMPLATES */

#endif /* EXPLICIT_TEMPLATE_INSTANTIATION */


#if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
template class ProtoVector<float, float, ImageSize>;
template void swap(ProtoVector<float, float, ImageSize>& u, ProtoVector<float, float, ImageSize>&);
template float inproduct(const ProtoVector<float, float, ImageSize>&, const ProtoVector<float, float, ImageSize>&);
template float MSE(const ProtoVector<float, float, ImageSize>&, const ProtoVector<float, float, ImageSize>&);
template float RMSE(const ProtoVector<float, float, ImageSize>&, const ProtoVector<float, float, ImageSize>&);
template float SNR(const ProtoVector<float, float, ImageSize>&, const ProtoVector<float, float, ImageSize>&);
template float PSNR(const ProtoVector<float, float, ImageSize>&, const ProtoVector<float, float, ImageSize>&);

template class Image<float, float>;
template class ImageRef<float, float>;
template class Subimage<float, float>;
template class ImageFileInterface<float, float>;
template ImageRef<float, float> operator+(const Image<float, float>&, const Image<float, float>&);
template ImageRef<float, float> operator+(const Image<float, float>&, float);
template ImageRef<float, float> operator-(const Image<float, float>&, const Image<float, float>&);
template ImageRef<float, float> operator-(const Image<float, float>&, float);
template ImageRef<float, float> operator-(float, const Image<float, float>&);
template ImageRef<float, float> operator*(const Image<float, float>&, float);
template ImageRef<float, float> operator*(float, const Image<float, float>&);
template ImageRef<float, float> operator/(const Image<float, float>&, float);
template ImageRef<float, float> abs(const Image<float, float>&);
template ImageRef<float, float> clip(const Image<float, float>&, float, float);
template ImageRef<float, float> isometry(const Image<float, float>&, unsigned char);
template ImageRef<float, float> subsample(const Image<float, float>&, unsigned int, unsigned int);
template ostream& operator<<(ostream &, const Image<float, float>&);

// Conversions from other types
#if !defined(NO_MEMBER_TEMPLATES)
template ProtoVector<float, float, ImageSize>::ProtoVector(const ProtoVector<unsigned char, real, ImageSize>&);
template ProtoVector<float, float, ImageSize>::ProtoVector(const ProtoVector<unsigned short, real, ImageSize>&);
template Image<float, float>::Image(const Image<unsigned char, real>&);
template Image<float, float>::Image(const Image<unsigned short, real>&);
#endif /* NO_MEMBER_TEMPLATES */

#endif /* EXPLICIT_TEMPLATE_INSTANTIATION */


