#!/usr/bin/perl

$conffile = "types.conf";

$types = {};
$headers = {};
$predecl = "";
$decl = "";
$declfile = "";
$postdecl = "";

readconf($conffile, $types, $headers, \$predecl, \$decl, \$postdecl, 
	 \$declfile);
writeheaders($types, $headers);
writedecl($types, $predecl, $decl, $postdecl, $declfile);
exit 0;

sub readconf {
  my ($conffile, $types, $headers, $predecl, $decl, $postdecl, $declfile) = @_;

  open(CONF,"<"."$conffile");
  while($line = <CONF>) {
    next if ($line =~ /^\#/);

    if ($line =~ /^\+header\s*\:\s*(\S+)/) {
      my $label = $1;
      $headers->{$label} = "";
      while ($line = <CONF>) { 
	if ($line !~ /^\-header/) {
	  $headers->{$label} .= $line;
	} else {
	  last;
	}
      }
      next;
    }

    if ($line =~ /^\+predecl/) {
      while ($line = <CONF>) {
	if ($line !~ /^\-predecl/) {
	  $$predecl .= $line;
	} else {
	  last;
	}
      }
      next;
    }

    if ($line =~ /^\+decl\s*\:\s*(\S+)/) {
      $$declfile = $1;
      while ($line = <CONF>) {
	if ($line !~ /^\-decl/) {
	  $$decl .= $line;
	} else {
	  last;
	}
      }
      next;
    }

    if ($line =~ /^\+postdecl/) {
      while ($line = <CONF>) {
	if ($line !~ /^\-postdecl/) {
	  $$postdecl .= $line;
	} else {
	  last;
	}
      }
      next;
    }

    if ($line =~ /\s*(\w+)\s*\=\s*([^\,]+)\,\s*([^\;]+)\;/) {
      my $trec = {};
      $trec->{"T"} = $2;
      $trec->{"R"} = $3;
      $types->{$1} = $trec;
    }
  }
  close(CONF);
}

sub writeheaders {
  my ($types, $headers) = @_;

  my ($label, $prfx, $file);
  foreach $prfx ( keys %$types ) {
    foreach $label ( keys %$headers ) {
      $file = $label;
      $file =~ s/\@P\@/$prfx/;
#      print $file . "\n";
      open(HDR,">"."$file");
      $text = $headers->{$label};
      $text = replace($types, $text, $prfx);
      print HDR $text;
      close(HDR);
    }
  }
}

sub writedecl {
  my ($types, $predecl, $decl, $postdecl, $declfile) = @_;

  my ($text, $prfx);
  open(DCL,">"."$declfile");
  $text = replace($types, $predecl);
  print DCL $text;

  foreach $prfx ( keys %$types ) {
    $text = replace($types, $decl, $prfx);
    print DCL $text;
  }

  $text = replace($types, $postdecl);
  print DCL $text;
  close(DCL);
}


sub replace {
  my ($types, $text, $prfx) = @_;

  my $rep = "";
  my ($line, $ml, $flag, $p, $t, $r, $mp, $mt, $mr);
  if (defined $prfx) {
    $t = $types->{$prfx}->{"T"};
    $r = $types->{$prfx}->{"R"};
  }
  while ($text =~ /^.*$/mg) {
    $line = $&;

    if ($line =~ /\@[PTR]\@/) {
      $line =~ s/\@P\@/$prfx/g;
      $line =~ s/\@T\@/$t/g;
      $line =~ s/\@R\@/$r/g;
      $flag = "true";
    } else {
      $flag = "false";
    }

     if ($line =~ /\$[PTR]\$/) {
      foreach $mp ( keys %$types ) {
	next if (defined $prfx && $flag eq "true" && $mp eq $prfx);
	$ml = $line;
	$mt = $types->{$mp}->{"T"};
	$mr = $types->{$mp}->{"R"};
	$ml =~ s/\$P\$/$mp/g;
	$ml =~ s/\$T\$/$mt/g;
	$ml =~ s/\$R\$/$mr/g;
	$rep .= $ml . "\n";
      }
      next;
    }

    $rep .= $line . "\n";
  }
  return $rep;
}
