// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

// $Id: LaplacianSQ.h,v 1.1 2003/02/01 17:46:23 brendt Exp $


#ifndef _LaplacianSQ_h
#define _LaplacianSQ_h

#include "SQuantise.h"


//! unit="Quantisation"
//! module="Scalar Quantisation" 
//! level = 2

//: Laplacian Scalar Quantiser

class LaplacianSQ : public ScalarQuantiser {
private:
  unsigned int uRate;
  unsigned int uRightLevels;
  float mu;
  float sigma;
  float *xk;

  static const float yk0[];
  static const float yk1[];
  static const float yk2[];
  static const float yk3[];
  static const float yk4[];
  static const float yk5[];
  static const float yk6[];
  static const float yk7[];
  static const float yk8[];
  static const float yk9[];
  static const float yk10[];
  static const float yk11[];
  static const float yk12[];
  static const unsigned int Rmax;
  static const float* yk[];
  
public:
  LaplacianSQ(float im = 0.0, float isigma = 1.0,
	      const QRate& qr = QRate(1));
  ~LaplacianSQ();
  //Reset pdf parameters
  void reset_shape(float im, float isigma);
  //Reset quantisation rate
  void reset_rate(const QRate& qr);
  int Index(float x) const;
  float Reconstruction(unsigned int idx) const;
};

#endif
