// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

// $Id: QLevels.h,v 1.1 2003/02/01 17:46:23 brendt Exp $

#ifndef _QLevels_h
#define _QLevels_h

//! unit="Quantisation"
//! module="Support" 
//! level = 1

//: The number of reconstruction levels in a quantiser
// QLevels and derived classes allow initialisation of quantiser classes
// without the ambiguity resulting from specifying bit rate,
// reconstruction levels and number of index bits as unsigned integers.

class QLevels {
private:
  unsigned int levels;

public:
  //!param: l  - Number of reconstruction levels
  QLevels(unsigned int l) { levels = l; }
  // Allow conversion from QLevels to an unsigned int representing
  // the number of reconstruction levels
  operator unsigned int (void) const { return levels; }
  
};


//: The bit rate of a quantiser
// QRate represents the actual bit rate of a quantiser

class QRate : public QLevels {
private:
  unsigned int uRate;
public:
  //!param: r  - Bit rate (must be an integral value)
  QRate(unsigned int r) : QLevels(1 << r) { uRate = r; }
  //: Return the bit rate
  unsigned int Rate(void) const { return uRate; }
};


//: The number of bits required for each index in a quantiser
//  QIndexBits represent the number of bits required to specify each
//  reconstruction level in a quantiser.  Note that QRate and
//  QIndexBits are equivalent for scalar quantisers

class QIndexBits : public QLevels {
private:
  unsigned int uIndexBits;
public:
  //!param: ib  - Number of bits per index (must be an integral value)
  QIndexBits(unsigned int ib) : QLevels(1 << ib) { uIndexBits = ib; }
  //: Return the number of bits per index
  unsigned int IndexBits(void) const { return uIndexBits; }
  
};

#endif
