// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Arguments.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

*/



#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <fstream>
#include <iomanip>
using std::cerr;
using std::setw;
using std::endl;
#else
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#endif

#include "Arguments.h"

void Arguments::error(char *prog, const CLArg* params, const CLArg* optparams,
		      const CLArg* flags)
{
  cerr << "Usage: " << prog << ' ';
  int i = 0;
  if (flags)
    while (flags[i].label)
      {
	char *flagname = new char [strlen(flags[i].label)+2];
	sprintf(flagname,"-%s",flags[i].label);
	cerr << "[ " << flagname;
	delete [] flagname;
	if (flags[i].description)
	  cerr << ' ' << flags[i].description;
	cerr << " ] ";
	i++;
      }
  i = 0;
  if (params)
    while (params[i].label)
      {
	cerr << params[i].description << ' ';
	i++;
      }
  i = 0;
  if (optparams)
    while (optparams[i].label)
      {
	cerr << "[ " << optparams[i].description << " ] ";
	i++;
      }
  cerr << endl;
  exit(1);
}


Arguments::Arguments(int argc, char* argv[], const CLArg* params,
	             const CLArg* optparams, const CLArg* flags)
{
  bool *done = new bool [argc];
  labels = new char * [argc];
  values = new char * [argc];
  for (int i = 0; i < argc; i++)
    {
      done[i] = false;
      labels[i] = 0;
      values[i] = 0;
    }

  int listpos = 0;

  //Work through flags
  if (flags)
    {
      int i = 0;
      while (flags[i].label)
	{
	  char *flagname = new char [strlen(flags[i].label)+2];
	  sprintf(flagname,"-%s",flags[i].label);
	  for (int j = 1; j < argc; j++)
	    if (!done[j] && strcmp(flagname, argv[j]) == 0)
	      {
		if (flags[i].description == 0)
		  {
		    labels[listpos++] = flags[i].label;
		    done[j] = true;
		    break;
		  }
		else
		  {
		    if (j+1 < argc && !done[j+1])
		      {
			labels[listpos] = flags[i].label;
			values[listpos++] = argv[j+1];
			done[j] = true;
			done[j+1] = true;
			break;
		      }
		    else
		      error(argv[0], params, optparams, flags);
		  }
	      }
	  delete [] flagname;
	  i++;
	}
    }

  //Do fixed position parameters
  if (params)
    {
      int i = 0, j = 1;
      while (params[i].label && j < argc)
	{
	  if (!done[j])
	    {
	      labels[listpos] = params[i].label;
	      values[listpos++] = argv[j];
	      done[j] = true;
	      i++;
	    }
	  j++;
	}
      if (params[i].label)
	error(argv[0], params, optparams, flags);
    }

  //Do optional parameters
  if (optparams)
    {
      int i = 0, j = 1;
      while (optparams[i].label && j < argc)
	{
	  if (!done[j])
	    {
	      labels[listpos] = optparams[i].label;
	      values[listpos++] = argv[j];
	      done[j] = true;
	      i++;
	    }
	  j++;
	}
    }
  for (int j = 1; j < argc; j++)
    if (!done[j])
      error(argv[0], params, optparams, flags);
  delete [] done;
}

Arguments::~Arguments()
{
  delete [] labels;
  delete [] values;
}

char *Arguments::arg(char *label)
{
  int i = 0;
  while (labels[i])
    {
      if (strcmp(labels[i], label) == 0)
	return values[i];
      i++;
    }
  return 0;
}

bool Arguments::value(char *label)
{
  int i = 0;
  while (labels[i])
    {
      if (strcmp(labels[i], label) == 0)
	return true;
      i++;
    }
  return false;
}

bool Arguments::value(char *label, char &fv)
{
  char *cp = arg(label);
  if (cp)
    {
      fv = *cp;
      return true;
    }
  else
    return false;
}

bool Arguments::value(char *label, unsigned int& fv)
{
  char *cp = arg(label);
  if (cp)
    {
      fv = (unsigned int)atol(cp);
      return true;
    }
  else
    return false;
}

bool Arguments::value(char *label, int& fv)
{
  char *cp = arg(label);
  if (cp)
    {
      fv = atoi(cp);
      return true;
    }
  else
    return false;
}

bool Arguments::value(char *label, float& fv)
{
  char *cp = arg(label);
  if (cp)
    {
      fv = atof(cp);
      return true;
    }
  else
    return false;
}

bool Arguments::value(char *label, double& fv)
{
  char *cp = arg(label);
  if (cp)
    {
      fv = atof(cp);
      return true;
    }
  else
    return false;
}


ostream& operator<<(ostream &s, const Arguments& a)
{
  int i = 0;
  while (a.labels[i])
    {
      s << setw(16) << a.labels[i]
	<< setw(32) << a.values[i] << endl;
      i++;
    }
  return s;
}
