// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Timer.cc,v 1.1 2003/02/01 17:46:23 brendt Exp $

*/

#include <string.h>
#include <math.h>
#include "Timer.h"

#if defined(HAVE_LIMITS_H)
#include <limits.h>
#endif

#if defined(HAVE_SYS_TIME_H)
#include <sys/time.h>
#endif

#if defined(HAVE_SYS_TIMES_H)
#include <sys/times.h>
#endif

#if defined(HAVE_SYS_TYPES_H)
#include <sys/types.h>
#endif

#if !defined(CLK_TCK)
#define CLK_TCK 3     //Kludge - not defined in SunOS
#endif

long ProcTime()
{
  tms *buffer = new tms;
  if (times(buffer) == -1)
    return -1;
  else
    return (buffer->tms_utime + buffer->tms_stime)/CLK_TCK;
}


Clock_CPU::Clock_CPU()
{
  clock();
}

void Clock_CPU::mark()
{
  initial = clock();
}

float Clock_CPU::time()
{
  return (float)clock()/CLOCKS_PER_SEC;
}

float Clock_CPU::interval()
{
  return (float)cinterval()/CLOCKS_PER_SEC;
}

clock_t Clock_CPU::ctime()
{
  return clock();
}

clock_t Clock_CPU::cinterval()
{
  return clock() - initial;
}

#if defined(HAVE_GETHRTIME)

Timer_Real::Timer_Real()
{
  reset();
}

void Timer_Real::reset()
{
  initial = gethrtime();
}

float Timer_Real::interval()
{
  return cinterval()*1e-9;
}

hrtime_t Timer_Real::cinterval()
{
  return gethrtime() - initial;
}

#endif

#if defined(HAVE_GETHRVTIME)

Timer_CPU::Timer_CPU()
{
  reset();
}

void Timer_CPU::reset()
{
  initial = gethrvtime();
}

float Timer_CPU::interval()
{
  return cinterval()*1e-9;
}

hrtime_t Timer_CPU::cinterval()
{
  return gethrvtime() - initial;
}

#endif


