// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*

  $Id: Timer.h,v 1.1 2003/02/01 17:46:23 brendt Exp $

*/

#ifndef _timer_h
#define _timer_h

#include "ImageLib.h"

#include <time.h>

#if defined(HAVE_SYS_TIME_H)
#include <sys/time.h>
#endif

//! unit="Miscellaneous"
//! module="Timers" 
//! level = 1

//:Return process CPU time
long ProcTime();

//:Timer - short periods ( < 36min), CPU time
class Clock_CPU {
private:
  clock_t initial;
public:
  Clock_CPU();
  //Set new time mark
  void mark();
  //Time in seconds since instantiation
  float time();
  //Time in seconds since last mark() call
  float interval();
  //Clock ticks since instantiation
  clock_t ctime();    
  //Clock ticks since last mark() call
  clock_t cinterval();
};

#if defined(HAVE_GETHRTIME)
//:Timer - Real time
class Timer_Real {
private:
  hrtime_t initial;
public:
  Timer_Real();
  void reset();
  float interval();
  hrtime_t cinterval();
};
#endif

#if defined(HAVE_GETHRVTIME)
//:Timer - CPU time
class Timer_CPU {
private:
  hrtime_t initial;
public:
  Timer_CPU();
  void reset();
  float interval();
  hrtime_t cinterval();
};
#endif
  
#endif



    
