// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: CoefAccess.cc,v 1.2 2005/10/06 14:46:06 brendt Exp $

 */

#include "CoefAccess.h"


CoefAccess::CoefAccess(floatVector& fv) : tree_vec(fv)
{
  float fdepth = log2(fv.vlength());
  assert( fdepth == floor(fdepth) );
  tree_depth = (unsigned int)fdepth;
}


floatSubvector CoefAccess::subband(unsigned int i)
{
  floatSubvector band(tree_vec, (1 << i), (1 << i) );
  return band;
}

CoefAccessSubtree CoefAccess::subtree(unsigned int i, unsigned int j)
{
  return CoefAccessSubtree(*this, i, j);
}

CoefAccessSubtree CoefAccess::subtree(unsigned int i, unsigned int j,
		                    unsigned int depth)
{
  return CoefAccessSubtree(*this, i, j, depth);
}


CoefAccessSubtree::CoefAccessSubtree(const CoefAccess& tree, unsigned int i,
			       unsigned int j) :
  ri(i), rj(j), ptree(tree)
{
  stdepth = tree.depth() - i;
}

CoefAccessSubtree::CoefAccessSubtree(const CoefAccess& tree, unsigned int i,
		               unsigned int j, unsigned int depth) :
  ri(i), rj(j), stdepth(depth), ptree(tree)
{ }


const CoefAccessSubtree& CoefAccessSubtree::operator=(const CoefAccessSubtree&
						      subtree)
{
  unsigned int src_depth = subtree.stdepth;
  unsigned int dst_depth = stdepth;
  assert( src_depth <= dst_depth );
  unsigned int i,j;
  for (i = 0; i < src_depth; i++)
    for (j = 0; j < CoefAccess::subband_length(i); j++)
      operator()(i, j) = subtree(i, j);
  return subtree;
}

const CoefTree& CoefAccessSubtree::operator=(const CoefTree& vecrep)
{
  unsigned int src_depth = vecrep.tree_depth;
  unsigned int dst_depth = stdepth;
  assert( src_depth <= dst_depth );
  unsigned int i,j;
  for (i = 0; i < src_depth; i++)
    for (j = 0; j < CoefAccess::subband_length(i); j++)
      operator()(i, j) = vecrep(i, j);
  return vecrep;
}
  
const CoefAccess& CoefAccessSubtree::operator=(const CoefAccess& mtree)
{
  unsigned int src_depth = mtree.depth();
  unsigned int dst_depth = stdepth;
  assert( src_depth <= dst_depth );
  unsigned int i,j;
  for (i = 0; i < src_depth; i++)
    for (j = 0; j < CoefAccess::subband_length(i); j++)
      operator()(i, j) = mtree(i, j);
  return mtree;
}

ostream& operator<<(ostream &s, const CoefAccessSubtree& t)
{
  s.setf(ios::fixed);
  s.setf(ios::showpoint);
  const unsigned int cols = 120;
  const unsigned int fieldw = 5;
  unsigned int i,j;
  for (i = 0; i < t.stdepth; i++)
    {
      unsigned int spacing = cols/(1 << (i+1));
      s << setw(spacing + fieldw/2) << setprecision(2) << t(i,0);
      for (j = 1; j < CoefAccess::subband_length(i); j++)
	s << setw(2*spacing + fieldw/2) << setprecision(2) << t(i,j);
      s << endl;
    }
		   
  return s;
}
