// -*- Mode:c++ -*-
/********************************COPYRIGHT*************************************
  
    This file forms a part of ImageLib, a C++ class library for image 
    processing.

    Copyright (C) 1998-2003 Brendt Wohlberg  <brendt@dip.ee.uct.ac.za>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

********************************COPYRIGHT*************************************/

/*
  
  $Id: NstdCoefAccess.cc,v 1.2 2005/10/06 14:46:06 brendt Exp $

 */

#include "NstdCoefAccess.h"


NstdCoefAccess::NstdCoefAccess(floatImage& fi) : tree_im(fi)
{
  assert( fi.iwidth() == fi.iheight() );
  float fdepth = log2(fi.iwidth());
  assert( fdepth == floor(fdepth) );
  tree_depth = (unsigned int)fdepth;
}


floatSubimage NstdCoefAccess::subband(unsigned int i, Direction drn)
{
  unsigned int c = 1 << i;
  unsigned int a = (drn == Horizontal)?0:c;
  unsigned int b = (drn == Vertical)?0:c;
  floatSubimage band(tree_im, a, b, c, c);
  return band;
}

NstdCoefAccessDrctSubtree NstdCoefAccess::subtree(unsigned int i,
						  unsigned int j,
						  unsigned int k,
						  Direction drn)
{
  return NstdCoefAccessDrctSubtree(*this, i, j, k, drn);
}

NstdCoefAccessDrctSubtree NstdCoefAccess::subtree(unsigned int i,
						  unsigned int j,
						  unsigned int k,
						  Direction drn,
						  unsigned int depth)
{
  return NstdCoefAccessDrctSubtree(*this, i, j, k, drn, depth);
}


NstdCoefAccessFullSubtree NstdCoefAccess::subtree(unsigned int i,
						  unsigned int j,
						  unsigned int k)
{
  return NstdCoefAccessFullSubtree(*this, i, j, k);
}

NstdCoefAccessFullSubtree NstdCoefAccess::subtree(unsigned int i,
						  unsigned int j,
						  unsigned int k,
						  unsigned int depth)
{
  return NstdCoefAccessFullSubtree(*this, i, j, k, depth);
}


NstdCoefAccessDrctSubtree::NstdCoefAccessDrctSubtree(const
						     NstdCoefAccess& tree,
						     unsigned int i,
						     unsigned int j,
						     unsigned int k,
					     NstdCoefAccess::Direction drn) :
  ri(i), rj(j), rk(k), stdrn(drn), ptree(tree)
{
  stdepth = tree.depth() - i;
}

NstdCoefAccessDrctSubtree::NstdCoefAccessDrctSubtree(const
						     NstdCoefAccess&
						     tree,
						     unsigned int i,
						     unsigned int j,
						     unsigned int k,
						NstdCoefAccess::Direction drn,
						     unsigned int depth) :
  ri(i), rj(j), rk(k), stdepth(depth), stdrn(drn), ptree(tree)
{ }

NstdCoefAccessDrctSubtree::NstdCoefAccessDrctSubtree(const
					     NstdCoefAccessFullSubtree& stree,
					     NstdCoefAccess::Direction drn) :
  ri(stree.ri), rj(stree.rj), rk(stree.rk),
  stdepth(stree.stdepth), stdrn(drn), ptree(stree.ptree)
{ }

const NstdCoefAccessDrctSubtree&
NstdCoefAccessDrctSubtree::operator=(const
				     NstdCoefAccessDrctSubtree& subtree)
{
  unsigned int src_depth = subtree.stdepth;
  unsigned int dst_depth = stdepth;
  assert( src_depth <= dst_depth );
  unsigned int i,j,k;
  for (i = 0; i < src_depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
	operator()(i, j, k) = subtree(i, j, k);
  return subtree;
}

const NstdCoefDrctTree& NstdCoefAccessDrctSubtree::operator=(const
						     NstdCoefDrctTree& vecrep)
{
  unsigned int src_depth = vecrep.tree_depth;
  unsigned int dst_depth = stdepth;
  assert( src_depth <= dst_depth );
  unsigned int i,j,k;
  for (i = 0; i < src_depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
	operator()(i, j, k) = vecrep(i, j, k);
  return vecrep;
}
  

NstdCoefAccessFullSubtree::NstdCoefAccessFullSubtree(const
						     NstdCoefAccess&
						     tree,
						     unsigned int i,
						     unsigned int j,
						     unsigned int k) :
  ri(i), rj(j), rk(k), ptree(tree)
{
  stdepth = tree.depth() - i;
}

NstdCoefAccessFullSubtree::NstdCoefAccessFullSubtree(const
						     NstdCoefAccess& tree,
						     unsigned int i,
						     unsigned int j,
						     unsigned int k,
						     unsigned int depth) :
  ri(i), rj(j), rk(k), stdepth(depth), ptree(tree)
{ }


const NstdCoefAccessFullSubtree&
NstdCoefAccessFullSubtree::operator=(const
				     NstdCoefAccessFullSubtree& subtree)
{
  int drn;
  for (drn = (int)NstdCoefAccess::Horizontal;
       drn <= (int)NstdCoefAccess::Diagonal; drn++)
    {
      NstdCoefAccessDrctSubtree srctree(subtree,
					(NstdCoefAccess::Direction)drn);
      NstdCoefAccessDrctSubtree dsttree(*this,
					(NstdCoefAccess::Direction)drn);
      dsttree = srctree;
    }
  return subtree;
}

const NstdCoefFullTree& NstdCoefAccessFullSubtree::operator=(const
						    NstdCoefFullTree& vecrep)
{
  unsigned int src_depth = vecrep.tree_depth;
  unsigned int dst_depth = stdepth;
  assert( src_depth <= dst_depth );
  unsigned int i,j,k,drn;
  for (drn = (unsigned int)NstdCoefAccess::Horizontal;
       drn <= (unsigned int)NstdCoefAccess::Diagonal; drn++)
    {
      for (i = 0; i < src_depth; i++)
	for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
	  for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
	    operator()(i, j, k, (NstdCoefAccess::Direction)drn)
	      = vecrep(i, j, k, (NstdCoefAccess::Direction)drn);
    }
  return vecrep;
}


NstdCoefDrctTree NstdCoefDrctTree::subtree(unsigned int ri,
					   unsigned int rj, unsigned int rk,
					   unsigned int depth) const
{
  NstdCoefDrctTree stree(depth);
  unsigned int i,j,k;
  for (i = 0; i < depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
	stree(i,j,k) = operator()(ri + i, rj*(1 << i) + j, rk*(1 << i) + k);
  return stree;
}


NstdCoefFullTree NstdCoefFullTree::subtree(unsigned int ri,
					   unsigned int rj, unsigned int rk,
					   unsigned int depth) const
{
  NstdCoefFullTree stree(depth);
  unsigned int i,j,k,drn;
  for (drn = (unsigned int)NstdCoefAccess::Horizontal;
       drn <= (unsigned int)NstdCoefAccess::Diagonal; drn++)
  for (i = 0; i < tree_depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
	stree(i,j,k,(NstdCoefAccess::Direction)drn) =
	  operator()(ri + i, rj*(1 << i) + j, rk*(1 << i) + k,
		     (NstdCoefAccess::Direction)drn);
  return stree;
}

NstdCoefDrctTree NstdCoefFullTree::subtree(unsigned int ri,
					   unsigned int rj, unsigned int rk,
					   NstdCoefAccess::Direction drn,
					   unsigned int depth) const
{
  NstdCoefDrctTree stree(depth);
  unsigned int i,j,k;
  for (i = 0; i < depth; i++)
    for (j = 0; j < NstdCoefAccess::subband_length(i); j++)
      for (k = 0; k < NstdCoefAccess::subband_length(i); k++)
	stree(i,j,k) = operator()(ri + i,
				  rj*(1 << i) + j, rk*(1 << i) + k,drn);
  return stree;
}
